-- 821 register with oe
library ieee;
use ieee.std_logic_1164.all;
library exemplar;
use exemplar.exemplar_1164.all;

ENTITY pulse_widths IS
    GENERIC (
        tlow: TIME := 0 ns;
        thigh: TIME := 0 ns);
    PORT (
        in0 : IN std_logic;
        out0: OUT std_logic);
END pulse_widths;

library ieee;
use ieee.std_logic_1164.all;
library exemplar;
use exemplar.exemplar_1164.all;

ARCHITECTURE behavior OF pulse_widths IS
BEGIN
    PROCESS (in0)
        VARIABLE prev_event : TIME := 0 ns;
    BEGIN
        IF prev_event /= 0 ns THEN
            IF in0 = '1' THEN
                ASSERT Now - prev_event > tlow
                    REPORT "Pulse width of '0' is less than " & time'image(tlow)
                    SEVERITY Warning;
            ELSIF in0 = '0' THEN
                ASSERT Now - prev_event > thigh
                    REPORT "Pulse width of '1' is less than " & time'image(thigh)
                    SEVERITY Warning;
            END IF;
        END IF;

        prev_event := Now;
    END PROCESS;

    out0 <= in0;
END behavior;

library IEEE;
use IEEE.std_logic_1164.all;
library exemplar;
use exemplar.exemplar_1164.all;

entity regoex is
generic ( tDQ :time := 10 ns; tEN : time := 12 ns; tDIS : time := 7 ns; tPLO : time := 7 ns; tPHI : time := 7 ns;
          tDS : time := 4 ns; tDH : time := 2 ns);
    port (
		ck : in std_logic;
        input : in STD_LOGIC;
        output : out STD_LOGIC;
        not_en: in STD_LOGIC
    );
end regoex;

library IEEE;
use IEEE.std_logic_1164.all;
library exemplar;
use exemplar.exemplar_1164.all;

architecture regoex_arch of regoex is
signal data : std_logic;
COMPONENT pulse_widths
    GENERIC (
        tlow: TIME := 0 ns;
        thigh: TIME := 0 ns);
    PORT (
        in0 : IN std_logic;
        out0: OUT std_logic);
END COMPONENT;

SIGNAL  tck : Std_logic; -- temps for pulse width checks
begin
-- ck to input setup and hold checks
l : ASSERT not((ck='1' and not ck'stable) and (not input'stable(tDS))) 
        REPORT "setup violation ck to input"
        SEVERITY warning;

m : ASSERT not((ck='1' and input'event) and (not ck'stable(tDH))) 
        REPORT "hold violation ck to input"
        SEVERITY warning;

-- pulse width minimums tests
pck: pulse_widths 
    GENERIC MAP (tlow => tPLO, thigh => tPHI)
    PORT MAP (in0 => ck, out0 => tck);

proc_en: process (ck,input,not_en,data)
begin
if (not_en='0') then
  if (rising_edge(ck)) then
    data <= input;
	output <= input	after tDQ;
  else
    output <= data after tEN;
  end if;
else
  if (rising_edge(ck)) then
    data <= input;
  end if;
  output <= 'Z' after tDIS;
end if;
end process proc_en;

end regoex_arch;

library IEEE;
use IEEE.std_logic_1164.all;
library exemplar;
use exemplar.exemplar_1164.all;


entity regoexn is
generic ( size :integer := 32;
          tDQ :time := 10 ns; tEN : time := 12 ns; tDIS : time := 7 ns;
         tPLO : time := 7 ns; tPHI : time := 7 ns;
          tDS : time := 4 ns; tDH : time := 2 ns);
    port (
		ck : in std_logic;
        input : in STD_LOGIC_VECTOR (size-1 downto 0);
        output : out STD_LOGIC_VECTOR (size-1 downto 0);
        not_en: in STD_LOGIC
    );
end regoexn;

library IEEE;
use IEEE.std_logic_1164.all;
library exemplar;
use exemplar.exemplar_1164.all;

architecture regoexn_arch of regoexn is
-- Component instantiated by Turbo autoplace on 11/20/95 at 11:49:16
COMPONENT regoex
 generic (  
          tDQ :time := 10 ns; tEN : time := 12 ns; tDIS : time := 7 ns;
         tPLO : time := 7 ns; tPHI : time := 7 ns;
          tDS : time := 4 ns; tDH : time := 2 ns);
    port (
		ck : in std_logic;
        input : in STD_LOGIC;
        output : out STD_LOGIC;
        not_en: in STD_LOGIC
    );
END COMPONENT;
begin

c_call: FOR i IN 0 TO size-1 GENERATE
          bits: regoex
          generic map (tDQ => tDQ, tDIS=>tDIS, tEN=>tEN, tPLO=>tPLO, tPHI=>tPHI,tDS=>tDS,tDH=>tDH) 
          port map (ck => ck, input => input(i), output => output(i), not_en => not_en);
        END GENERATE;    
end regoexN_arch;
